@@ -32,12 +32,12 @@ class HDUBase(object):
     """
     def __init__(self, fits, ext, **keys):
 
-        if not keys:
+        if keys:
             import warnings
             warnings.warn(
                 "The keyword arguments '%s' are being ignored! This warning "
-                "will be an error in a future version of `fitsio`!",
-                DeprecationWarning)
+                "will be an error in a future version of `fitsio`!" % keys,
+                DeprecationWarning, stacklevel=2)
 
         self._FITS = fits
         self._ext = ext
